package com.logntw.eva.jdbc;

import java.math.BigDecimal;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.Ref;
import java.sql.RowId;
import java.sql.SQLXML;
import java.sql.Struct;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;


//should merge with JavaType!!
//O JDBCClazz?  X DBType?
public enum JDBCClazz
{
	
	BIT(Types.BIT, boolean.class), 
	TINYINT(Types.TINYINT, byte.class), 
	SMALLINT(Types.SMALLINT, short.class), 
	INTEGER(Types.INTEGER, int.class), 
	BIGINT(Types.BIGINT, long.class), 
	FLOAT(Types.FLOAT, double.class),
	REAL(Types.REAL, float.class), 
	DOUBLE(Types.DOUBLE, double.class), 
	NUMERIC(Types.NUMERIC, BigDecimal.class), 
	DECIMAL(Types.DECIMAL, BigDecimal.class),
	VARCHAR(Types.VARCHAR, String.class), 
	CHAR(Types.CHAR, String.class), 	
	LONGVARCHAR(Types.LONGVARCHAR, String.class), 
	DATE(Types.DATE, Date.class), 
	TIME(Types.TIME, Time.class), 
	TIMESTAMP(Types.TIMESTAMP, Timestamp.class),
	VARBINARY(Types.VARBINARY, byte[].class), 
	BINARY(Types.BINARY, byte[].class), 	
	LONGVARBINARY(Types.LONGVARBINARY, byte[].class),
	BLOB(Types.BLOB, Blob.class), 
	CLOB(Types.CLOB, Clob.class),
	BOOLEAN(Types.BOOLEAN, boolean.class),
	NCHAR(Types.NCHAR, String.class), 
	NVARCHAR(Types.NVARCHAR, String.class), 
	LONGNVARCHAR(Types.LONGNVARCHAR, String.class), 
	NCLOB(Types.NCLOB, NClob.class), 
	ARRAY(Types.ARRAY, Array.class), 
	ROWID(Types.ROWID, RowId.class), 
	REF(Types.REF, Ref.class), 
	NULL(Types.NULL, null), 
	OTHER(Types.OTHER, null), 
	JAVA_OBJECT(Types.JAVA_OBJECT, Class.class),
	DISTINCT(Types.DISTINCT, Object.class),	
	STRUCT(Types.STRUCT, Struct.class),	
	DATALINK(Types.DATALINK, java.net.URL.class),  		
	SQLXML(Types.SQLXML, SQLXML.class), 
	
	;
	
	@Override
	public String toString()
	{
		return this.name();
	}
	
	public final int		TYPE_CODE;
	public final Class<?>	JAVA_TYPE;
	public final boolean 	IS_PRIMITIVE;

	private JDBCClazz(int TYPE_CODE, Class<?> JAVA_TYPE)
	{				
		this.TYPE_CODE = TYPE_CODE;
		this.JAVA_TYPE = JAVA_TYPE;
		this.IS_PRIMITIVE = JAVA_TYPE == null ? false : JAVA_TYPE.isPrimitive();
	}

	private static final Object SYNC = new Object();
	static
	{
		init();
	}	

	private static Map<Integer, JDBCClazz>	codeToType; 
	private static Map<Class<?>, JDBCClazz[]>	clzToTypes;		
	public static void init()
	{
		if(codeToType == null)
		{			
			synchronized(SYNC)
			{				
				if(codeToType == null)
				{
					codeToType = new HashMap<Integer, JDBCClazz>();
					Map<Class<?>, List<JDBCClazz>> clzToTypes_ = new IdentityHashMap<Class<?>, List<JDBCClazz>>();	

					for (JDBCClazz type : JDBCClazz.values())
					{
						codeToType.put(type.TYPE_CODE, type);
						if(type.JAVA_TYPE == null)
							continue;						
						List<JDBCClazz> list = clzToTypes_.get(type.JAVA_TYPE);
						if(list == null)
						{
							list = new ArrayList<JDBCClazz>();
							clzToTypes_.put(type.JAVA_TYPE, list);
						}
						list.add(type);												
					}
					
					clzToTypes = new IdentityHashMap<Class<?>, JDBCClazz[]>();				
					for(Map.Entry<Class<?>, List<JDBCClazz>> entry : clzToTypes_.entrySet())
					{
						JDBCClazz[] arr = new JDBCClazz[entry.getValue().size()];
						entry.getValue().toArray(arr);
						clzToTypes.put(entry.getKey(), arr);
					}															
				}
			}
		}
	}

	public final static JDBCClazz forCode(int code)
	{
		return codeToType.get(code);
	}
	
	public static Class<?> primeType(Class<?> type)
	{
		if(type==Byte.class)
			return byte.class;
		if(type==Integer.class)
			return int.class;
		if(type==Long.class)
			return long.class;
		if(type==Double.class)
			return double.class;
		if(type==Float.class)
			return float.class;		
		if(type==Character.class)
			return char.class;
		if(type==Short.class)
			return short.class;		
		if(type==Boolean.class)
			return boolean.class;
		if(type==Void.class)
			return void.class;
		return type;
	}
	
	public static JDBCClazz getIntegerClzByPrecision(int precision)
	{
		if(precision >= 19)
			return BIGINT;
		if(precision >= 10)
			return INTEGER;
		if(precision >= 5)
			return SMALLINT;
		if(precision >= 3)
			return TINYINT;
		if(precision >= 1)
			return BIT;
		return INTEGER;
	}
	
	public final static JDBCClazz[] forClz(Class<?> clz)
	{
		return clzToTypes.get(primeType(clz));
	}
}
