package com.logntw.eva.orm;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Savepoint;

import com.logntw.eva.orm.imp.DefoConnectionProviderImpFactory;
import com.logntw.eva.orm.imp.DBConnectionProviderImp;
import com.logntw.eva.orm.imp.DBConnectionProviderImpFactory;



public abstract class AbstractConnectionProvider extends AbstractManagedDBCProvider implements DBConnectionProvider
{
	public abstract String connectionStrPrifix();
	public abstract String connectionStrPostfix();
	public abstract String driverClassName();
		
	protected final DBConnectionProviderImp imp;
	
	public AbstractConnectionProvider(String databaseName, String id, String password) throws SQLException
	{
		this(DefoConnectionProviderImpFactory.getInstance(), databaseName, id, password);
	}
	
	public AbstractConnectionProvider(String databaseNameOrURI) throws SQLException
	{
		this(DefoConnectionProviderImpFactory.getInstance(), databaseNameOrURI);
	}
	
	public AbstractConnectionProvider(DBConnectionProviderImpFactory factory, String databaseName, String id, String password) throws SQLException
	{
		imp = factory.getInstance(this, databaseName, id, password);
	}
	
	public AbstractConnectionProvider(DBConnectionProviderImpFactory factory, String databaseNameOrURI) throws SQLException
	{
		imp = factory.getInstance(this, databaseNameOrURI);
	}
	
	@Override
	public Connection getConnection()throws SQLException	
	{		
		Connection conn = imp.getConnection();
		conn.setAutoCommit(false);
		return conn;
	}
	
	@Override
	public boolean commit(Connection connection) throws SQLException
	{
		return imp.commit(connection);	
	}

	@Override
	public boolean rollback(Connection connection) throws SQLException
	{
		return imp.rollback(connection);	
	}

	@Override
	public boolean rollback(Connection connection, Savepoint savepoint) throws SQLException
	{
		return imp.rollback(connection, savepoint);	
	}
	
	@Override
	public boolean releaseConnection(Connection connection) throws SQLException
	{
		return imp.releaseConnection(connection);
	}
	
	@Override
	public void dispose()
	{
		imp.dispose();
	}
}
