package com.logntw.eva.orm.imp;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Savepoint;

import com.jolbox.bonecp.BoneCP;
import com.jolbox.bonecp.BoneCPConfig;
import com.logntw.eva.orm.AbstractConnectionProvider;
import com.logntw.eva.orm.DBConnectionProvider;

public final class BoneCPConnectionProviderImpFactory implements DBConnectionProviderImpFactory
{
	private final static class BoneCPConnectionProviderImp implements DBConnectionProviderImp
	{
		private final BoneCP connectionPool;
		
		private BoneCPConnectionProviderImp(AbstractConnectionProvider provider, String databaseName, String id, String password) throws SQLException
		{
			if(databaseName == null || id == null || password == null) 
				throw new java.lang.IllegalArgumentException("[oCIBr1001]");
			
			String uri = databaseName.startsWith("jdbc:") ? databaseName : new StringBuffer(provider.connectionStrPrifix()).append(databaseName).append(provider.connectionStrPostfix()).toString();
			BoneCPConfig config = new BoneCPConfig();    // create a new configuration object
	        // set the JDBC url
	        config.setJdbcUrl(uri);
	        config.setUsername(id);            // set the username
	        config.setPassword(password);                // set the password
	        
	        connectionPool = new BoneCP(config);     // setup the connection pool        
		}
		
		private BoneCPConnectionProviderImp(String URI) throws SQLException
		{
			if(URI == null) 
				throw new java.lang.IllegalArgumentException("[oCIBr1002]");
						
			BoneCPConfig config = new BoneCPConfig();    // create a new configuration object
	        // set the JDBC url
	        config.setJdbcUrl(URI);
	        
	        connectionPool = new BoneCP(config);     // setup the connection pool        
		}
		
		@Override
		public Connection getConnection()throws SQLException	
		{		
			return connectionPool.getConnection();
		}
		
		@Override
		public boolean commit(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection, Savepoint savepoint) throws SQLException
		{
			// do nothing.
			return true;
		}
		
		@Override
		public boolean releaseConnection(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}
	
		public void dispose()
		{
			connectionPool.shutdown();
		}
	}
	
	@Override
	public DBConnectionProviderImp getInstance(AbstractConnectionProvider provider, String databaseName, String id,
			String password) throws SQLException
	{
		return new BoneCPConnectionProviderImp(provider, databaseName, id, password);
	}

	@Override
	public DBConnectionProviderImp getInstance(AbstractConnectionProvider provider, String databaseNameOrURI) throws SQLException
	{
		return databaseNameOrURI.startsWith("jdbc:") ? new BoneCPConnectionProviderImp(databaseNameOrURI) : new BoneCPConnectionProviderImp(provider, databaseNameOrURI, "sa", "");
	}
	
	private final static BoneCPConnectionProviderImpFactory INSTANCE = new BoneCPConnectionProviderImpFactory();
	public static BoneCPConnectionProviderImpFactory getInstance()
	{
		return INSTANCE;
	}

	@Override
	public DBConnectionProviderImp getInstance(DBConnectionProvider provider, String URI, String driverClassName)
			throws SQLException
	{
		return new BoneCPConnectionProviderImp(URI);
	}
}
