package com.logntw.eva.orm.imp;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Savepoint;

import com.logntw.eva.orm.AbstractConnectionProvider;
import com.logntw.eva.orm.DBConnectionProvider;


public final class DefoConnectionProviderImpFactory implements DBConnectionProviderImpFactory
{
	private final static class DefoConnectionProviderImp implements DBConnectionProviderImp
	{
		private final String uri;
		private final String id;
		private final String password;
		private DefoConnectionProviderImp(AbstractConnectionProvider provider, String databaseName, String id, String password)
		{
			if(databaseName == null || id == null || password == null) 
				throw new java.lang.IllegalArgumentException("[oCIAr1001]");
			this.uri = databaseName.startsWith("jdbc:") ? databaseName : new StringBuffer(provider.connectionStrPrifix()).append(databaseName).append(provider.connectionStrPostfix()).toString();
			this.id = id;
			this.password = password;
		}
		
		private DefoConnectionProviderImp(AbstractConnectionProvider provider, String databaseName)
		{
			this(provider, databaseName, "sa", "");
		}
	
		@Override
		public Connection getConnection() throws SQLException
		{
			return DriverManager.getConnection(uri, id, password);		
		}
		
		@Override
		public boolean commit(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection, Savepoint savepoint) throws SQLException
		{
			// do nothing.
			return true;
		}
		
		@Override
		public boolean releaseConnection(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}
	
		@Override
		public void dispose()
		{
			// do nothing.
		}	
	}
	
	private final static class DefoConnectionProviderImp2 implements DBConnectionProviderImp
	{
		private final String uri;	
		private DefoConnectionProviderImp2(String URI)
		{
			if(URI == null)
				throw new java.lang.IllegalArgumentException("[oCIAr1002]");
			this.uri = URI;
		}
	
		@Override
		public Connection getConnection() throws SQLException
		{
			return DriverManager.getConnection(uri);		
		}
		
		@Override
		public boolean commit(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection, Savepoint savepoint) throws SQLException
		{
			// do nothing.
			return true;
		}
	
		@Override
		public boolean releaseConnection(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}
		
		@Override
		public void dispose()
		{
			// do nothing.
		}
	}
	
	@Override
	public DBConnectionProviderImp getInstance(AbstractConnectionProvider provider, String databaseName, String id,
			String password)
	{
		return new DefoConnectionProviderImp(provider, databaseName, id, password);
	}

	@Override
	public DBConnectionProviderImp getInstance(AbstractConnectionProvider provider, String databaseNameOrURI)
	{
		return databaseNameOrURI.startsWith("jdbc:") ? new DefoConnectionProviderImp2(databaseNameOrURI) : new DefoConnectionProviderImp(provider, databaseNameOrURI);
	}
	
	private final static DefoConnectionProviderImpFactory INSTANCE = new DefoConnectionProviderImpFactory();
	public static DefoConnectionProviderImpFactory getInstance()
	{
		return INSTANCE;
	}

	@Override
	public DBConnectionProviderImp getInstance(DBConnectionProvider provider, String URI, String driverClassName)
			throws SQLException
	{		
		return new DefoConnectionProviderImp2(URI);
	}
}
