package com.logntw.eva.orm.imp;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Savepoint;

import org.apache.tomcat.jdbc.pool.DataSource;
import org.apache.tomcat.jdbc.pool.PoolProperties;

import com.logntw.eva.orm.AbstractConnectionProvider;
import com.logntw.eva.orm.DBConnectionProvider;

public class TomcatJDBCConnectionProviderImpFactory implements DBConnectionProviderImpFactory
{
	private final static class TomcatJDBCConnectionProviderImp implements DBConnectionProviderImp
	{
		private final DataSource	datasource;

		private TomcatJDBCConnectionProviderImp(AbstractConnectionProvider provider, String databaseName, String id,
				String password) throws SQLException
		{
			if (databaseName == null || id == null || password == null)
				throw new java.lang.IllegalArgumentException("[oCIBr1001]");

			String uri = databaseName.startsWith("jdbc:") ? databaseName : new StringBuffer(provider.connectionStrPrifix()).append(databaseName).append(
					provider.connectionStrPostfix()).toString();
			PoolProperties p = new PoolProperties();
			p.setDriverClassName(provider.driverClassName());
			p.setUrl(uri);
			p.setUsername(id);
			p.setPassword(password);

			datasource = new DataSource();
			datasource.setPoolProperties(p);			
		}
		
		private TomcatJDBCConnectionProviderImp(AbstractConnectionProvider provider, String URI) throws SQLException
		{
			if (URI == null)
				throw new java.lang.IllegalArgumentException("[oCIBr1002]");
			
			PoolProperties p = new PoolProperties();
			p.setDriverClassName(provider.driverClassName());
			p.setUrl(URI);

			datasource = new DataSource();
			datasource.setPoolProperties(p);			
		}
		
		private TomcatJDBCConnectionProviderImp(DBConnectionProvider provider, String URI, String driverClassName) throws SQLException
		{
			if (URI == null)
				throw new java.lang.IllegalArgumentException("[oCIBr1003]");
			
			PoolProperties p = new PoolProperties();
			p.setDriverClassName(driverClassName);
			p.setUrl(URI);

			datasource = new DataSource();
			datasource.setPoolProperties(p);			
		}

		@Override
		public Connection getConnection() throws SQLException
		{
			return datasource.getConnection();
		}
		
		@Override
		public boolean commit(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		@Override
		public boolean rollback(Connection connection, Savepoint savepoint) throws SQLException
		{
			// do nothing.
			return true;
		}
		
		@Override
		public boolean releaseConnection(Connection connection) throws SQLException
		{
			// do nothing.
			return true;
		}

		public void dispose()
		{
			datasource.close();
		}
	}
		
	@Override
	public DBConnectionProviderImp getInstance(AbstractConnectionProvider provider, String databaseName,
			String id, String password) throws SQLException
	{
		return new TomcatJDBCConnectionProviderImp(provider, databaseName, id, password);
	}

	@Override
	public DBConnectionProviderImp getInstance(AbstractConnectionProvider provider, String databaseNameOrURI)
			throws SQLException
	{
		return databaseNameOrURI.startsWith("jdbc:") ? new TomcatJDBCConnectionProviderImp(provider, databaseNameOrURI) : new TomcatJDBCConnectionProviderImp(provider, databaseNameOrURI, "sa", "");
	}
	
	private final static TomcatJDBCConnectionProviderImpFactory INSTANCE = new TomcatJDBCConnectionProviderImpFactory();
	public static TomcatJDBCConnectionProviderImpFactory getInstance()
	{
		return INSTANCE;
	}

	@Override
	public DBConnectionProviderImp getInstance(DBConnectionProvider provider, String URI, String driverClassName) throws SQLException
	{
		return new TomcatJDBCConnectionProviderImp(provider, URI, driverClassName);
	}
}
