package com.logntw.eva.sql.dialect;

import com.logntw.eva.jdbc.JDBCClazz;


public class DB2_Dialect extends Dialect
{
	public static final DB2_Dialect INSTANCE = new DB2_Dialect();
	
	public DB2_Dialect()
	{			
		indexable( JDBCClazz.BIT, "smallint" );
		indexable( JDBCClazz.BIGINT, "bigint" );
		indexable( JDBCClazz.SMALLINT, "smallint" );
		indexable( JDBCClazz.TINYINT, "smallint" );
		indexable( JDBCClazz.INTEGER, "integer" );
		indexable( JDBCClazz.CHAR, "char(1)" );
		indexable( JDBCClazz.VARCHAR, "varchar(N)" );
		indexable( JDBCClazz.FLOAT, "float" );
		indexable( JDBCClazz.DOUBLE, "double" );
		indexable( JDBCClazz.DATE, "date" );
		indexable( JDBCClazz.TIME, "time" );
		indexable( JDBCClazz.TIMESTAMP, "timestamp" );
		indexable( JDBCClazz.VARBINARY, "varchar(N) for bit data" );
		indexable( JDBCClazz.NUMERIC, "numeric(N,M)" );
		notIndexable( JDBCClazz.BLOB, "blob(N)" );
		notIndexable( JDBCClazz.CLOB, "clob(N)" );
		notIndexable( JDBCClazz.LONGVARCHAR, "long varchar" );
		notIndexable( JDBCClazz.LONGVARBINARY, "long varchar for bit data" );					
	}

	@Override
	public String name()
	{		
		return "db2";
	}
	
	@Override
	public String getIdentityColumnString()
	{
		return "generated by default as identity"; //not null ... (start with 1) is implicit
	}	
	
	@Override
	public boolean supportsLimit() 
	{
		return true;
	}
	
	private static boolean hasDistinct(String lowCaseSelectStmt) 
	{
		return lowCaseSelectStmt.indexOf("select distinct")>=0;		
	}
	
	private static StringBuilder getRowNumber(String lowCaseSelectStmt) 
	{				
		StringBuilder rownumber = new StringBuilder(50)
		.append(", rownumber() over(");

		int orderByIndex = lowCaseSelectStmt.lastIndexOf("order by");
		if ( orderByIndex > 0 && !hasDistinct(lowCaseSelectStmt)) 
		{
			rownumber.append(lowCaseSelectStmt.substring(orderByIndex));
		}
		rownumber.append(") as rownumber_");
		return rownumber;
	}

	@Override
	public StringBuilder getLimitStringEnd(StringBuilder sqlstmt) 
	{		
		String lowCaseSelectStmt = sqlstmt.toString().toLowerCase();
		StringBuilder pagingSelect = new StringBuilder( sqlstmt.length() + 100 )			
				.append("select * from ( select "); 			// nest the main query in an outer select
										
		if ( hasDistinct(lowCaseSelectStmt) ) 
		{
			pagingSelect.append(" row_.* ").append(getRowNumber(lowCaseSelectStmt)).append(" from ")			// add another (inner) nested select
					.append( sqlstmt ) // add the main query // add the rownnumber bit into the outer query select list 
					.append(" ) as row_"); 					// close off the inner nested select
		}
		else 
		{
			int from = lowCaseSelectStmt.indexOf(" from ");
			pagingSelect.append( sqlstmt.insert(from, getRowNumber(lowCaseSelectStmt)).substring(6) ); // add the main query // add the rownnumber bit into the outer query select list 						
		}
		
		pagingSelect.append(" ) as temp_ where rownumber_ between ?+1 and ?");
		return pagingSelect;
	}
		
	@Override
	public boolean useMaxForLimit() 
	{
		return true;
	}
		
	@Override
	public String getWriteLockString(int timeout) 
	{
		return " for read only with rs";
	}
}
