package com.logntw.eva.sql.dialect;

import java.util.HashMap;
import java.util.Map;

import com.logntw.eva.jdbc.JDBCClazz;
import com.logntw.eva.jdbc.SQLType;

public abstract class Dialect
{	
	Dialect(){}
			
	protected final Map<String, JDBCClazz> jdbcTypes = new HashMap<String, JDBCClazz>();
	@SuppressWarnings("serial")
	protected final Map<JDBCClazz, SQLType[]> sqlTypes = new HashMap<JDBCClazz, SQLType[]>()
	{
		@Override
		public SQLType[] put(JDBCClazz key, SQLType[] value)
		{
			for(SQLType str : value)
				jdbcTypes.put(str.name(), key);
			return super.put(key, value);
		}
	};
		
	private void indexable(boolean indexable, JDBCClazz jdbcType, String...sqlTypeNames)
	{
		int len = sqlTypeNames.length;
		SQLType[] types = new SQLType[len];
		for(int i = 0 ; i < len ; i ++)
			types[i] = new SQLType(indexable, sqlTypeNames[i].toUpperCase());
		this.sqlTypes.put(jdbcType, types);
	}
	
	protected final void indexable(JDBCClazz jdbcType, String...sqlTypeNames)
	{
		indexable(true, jdbcType, sqlTypeNames);
	}
	
	protected final void notIndexable(JDBCClazz jdbcType, String...sqlTypeNames)
	{
		indexable(false, jdbcType, sqlTypeNames);
	}
		
	public final SQLType[] getSQLTypes(JDBCClazz jdbcType)
	{		
		return sqlTypes.get(jdbcType);
	}

	
	public final JDBCClazz getJDBCType(String sqlType)
	{
		sqlType = sqlType.toUpperCase();
		return jdbcTypes.get(sqlType);
	}
	
	public abstract String name();
					
	public String notNullStmt()
	{
		return "NOT NULL";
	}
	
	public String getIdentityColumnString()
	{
		throw new UnsupportedOperationException( "[dCIDt1001]Dialect does not support identity key generation" );
	}	
	
	// limit
	public boolean supportsLimit()
	{
		return false;
	}

	public StringBuilder getLimitStringStart(StringBuilder selectStmt)
	{
		if(!supportsLimit())
			throw new UnsupportedOperationException( "[dCIDt1002]paged query is not supported" );
		return selectStmt;
	}
	
	public StringBuilder getLimitStringEnd(StringBuilder sqlstmt)
	{
		if(!supportsLimit())
			throw new UnsupportedOperationException( "[dCIDt1003]paged query is not supported" );
		return sqlstmt;
	}

	// Does the LIMIT clause come at the start of the SELECT statement, rather than at the end?
	public boolean bindLimitParametersFirst()
	{
		return false;
	}
	
	public boolean bindLimitParametersInReverseOrder() 
	{
		return false;
	}
	
	// Does the LIMIT clause take a "maximum" row number instead of a total number of returned rows?
	// True if limit is relative from offset; false otherwise.
	public boolean useMaxForLimit()
	{
		return false;
	}			
	
	// select for update
	// getForUpdateString
	public String getWriteLockString(int timeout) 
	{
		return " for update";
	}

	public String getReadLockString(int timeout) 
	{
		return getWriteLockString(timeout);
	}
	
	public enum ReturnGeneratedKeys
	{
		NEVER,
		INSERT_ONE,
		INSERT
	}
	
	// added.
	public ReturnGeneratedKeys returnGeneratedKeys()
	{
		return ReturnGeneratedKeys.INSERT_ONE;
	}
	
	public enum SubConditionSQL 
	{
		JOIN_FIRST,
		JOIN_LAST,
		EXIST,
		IN,	
	}	
	
	public SubConditionSQL updateWithSubCondition()
	{
		return SubConditionSQL.IN;
	}
	
	public SubConditionSQL deleteWithSubCondition()
	{
		return SubConditionSQL.EXIST;
	}
	
	public boolean putKeywordAsAheadAlias() // Oracle
	{
		return true;
	}
}
