package com.logntw.eva.sql.dialect;

import com.logntw.eva.jdbc.JDBCClazz;

public final class H2Dialect extends Dialect
{
	public static final H2Dialect INSTANCE = new H2Dialect();
	
	public H2Dialect()
	{			
		indexable( JDBCClazz.BOOLEAN, "boolean" );
		indexable( JDBCClazz.BIGINT, "bigint" );
		indexable( JDBCClazz.BINARY, "binary" );
		indexable( JDBCClazz.BIT, "boolean" );
		indexable( JDBCClazz.CHAR, "char(N)" );
		indexable( JDBCClazz.DATE, "date" );
		indexable( JDBCClazz.DECIMAL, "decimal(N,M)" );
		indexable( JDBCClazz.NUMERIC, "decimal(N,M)" );
		indexable( JDBCClazz.DOUBLE, "double" );
		indexable( JDBCClazz.FLOAT, "float" );
		indexable( JDBCClazz.INTEGER, "integer" );
		notIndexable( JDBCClazz.LONGVARBINARY, "longvarbinary" );
		notIndexable( JDBCClazz.LONGVARCHAR, "longvarchar" );
		indexable( JDBCClazz.REAL, "real" );
		indexable( JDBCClazz.SMALLINT, "smallint" );
		indexable( JDBCClazz.TINYINT, "tinyint" );
		indexable( JDBCClazz.TIME, "time" );
		indexable( JDBCClazz.TIMESTAMP, "timestamp" );
		indexable( JDBCClazz.VARCHAR, "varchar(N)" );
		indexable( JDBCClazz.VARBINARY, "binary(N)" );
		notIndexable( JDBCClazz.BLOB, "blob" );
		notIndexable( JDBCClazz.CLOB, "clob" );			
	}

	@Override
	public String name()
	{		
		return "h2";
	}
	
	@Override
	public String getIdentityColumnString()
	{
		return "generated by default as identity"; //not null ... (start with 1) is implicit
	}
	
	@Override
	public boolean supportsLimit() 
	{
		return true;
	}
	
	@Override
	public StringBuilder getLimitStringEnd(StringBuilder sqlStmt) 
	{
		return sqlStmt.append(" limit ? offset ?");		
	}

	@Override
	public boolean bindLimitParametersInReverseOrder() 
	{
		return true;
	}
}
