package com.logntw.eva.sql.dialect;

import com.logntw.eva.jdbc.JDBCClazz;



public class Oracle_10_Dialect extends Dialect
{	
	public static final Oracle_10_Dialect INSTANCE = new Oracle_10_Dialect();
	
	public Oracle_10_Dialect()
	{				
		indexable( JDBCClazz.CHAR, "char(1 char)" );
		indexable( JDBCClazz.VARCHAR, "varchar2((N) char)", "long");		
		
		indexable( JDBCClazz.BIT, "number(1,0)" );
		indexable( JDBCClazz.BIGINT, "number(19,0)" );
		indexable( JDBCClazz.SMALLINT, "number(5,0)" );
		indexable( JDBCClazz.TINYINT, "number(3,0)" );
		indexable( JDBCClazz.INTEGER, "number(10,0)" );

		indexable( JDBCClazz.FLOAT, "float" );
		indexable( JDBCClazz.DOUBLE, "double precision" );
		indexable( JDBCClazz.NUMERIC, "number(N,M)" );
		indexable( JDBCClazz.DECIMAL, "number(N,M)" );
		
		indexable( JDBCClazz.DATE, "date" );
		indexable( JDBCClazz.TIME, "date" );
		indexable( JDBCClazz.TIMESTAMP, "date" );
				
		notIndexable( JDBCClazz.BINARY, "raw(N)", "long raw" );
		notIndexable( JDBCClazz.VARBINARY, "raw(N)", "long raw" );

		notIndexable( JDBCClazz.BLOB, "blob" );
		notIndexable( JDBCClazz.CLOB, "clob" );

		notIndexable( JDBCClazz.LONGVARCHAR, "long" );
		notIndexable( JDBCClazz.LONGVARBINARY, "long raw" );			
	}

	@Override
	public String name()
	{		
		return "oracle10";
	}
	
	// means no stmt to append.
	// and needs to fill something when inserting.
	@Override
	public String getIdentityColumnString()
	{
		//return "[Please crate a 'Sequence' and indicate it by '@AutoGenerated(generator=\"...\")']";
		return "";
	}
	
	@Override
	public boolean supportsLimit()
	{
		return true;
	}
	
	@Override
	public StringBuilder getLimitStringEnd(StringBuilder sqlStmt) 
	{
		// will never occur, because we process lock after process limit.
		// if it is only for self-generated SQL, it can be more simplified.
		
		String lowCaseSqlStmt = sqlStmt.toString().toLowerCase();
		String forUpdateClause = null;						
		final int forUpdateIndex = lowCaseSqlStmt.lastIndexOf( "for update") ;
		if ( forUpdateIndex > -1 ) 
		{
			// save 'for update ...' and then remove it
			forUpdateClause = sqlStmt.substring( forUpdateIndex - 1);
			sqlStmt.delete(forUpdateIndex, sqlStmt.length());						
		}
		
		sqlStmt.insert(0, "select * from ( select row_.*, rownum rownum_ from ( ").append(" ) row_ where rownum <= ?) where rownum_ > ?");
		
		if ( forUpdateIndex > -1 ) 		
			sqlStmt.append( forUpdateClause );
				
		return sqlStmt;
	}
	
	@Override
	public boolean bindLimitParametersInReverseOrder() 
	{
		return true;
	}
	
	@Override
	public boolean useMaxForLimit() 
	{
		return true;
	}
		
	@Override
	public String getWriteLockString(int timeout) 
	{
		if ( timeout < 0) 
		{
			return " for update nowait";
		}
		else if ( timeout > 0 ) 
		{
			// convert from milliseconds to seconds
			float seconds = timeout / 1000.0f;
			timeout = Math.round(seconds);
			return " for update wait " + timeout;
		}
		else
			return " for update";
	}
	
	@Override
	public boolean putKeywordAsAheadAlias() // Oracle
	{
		return false;
	}
	
	@Override
	public SubConditionSQL deleteWithSubCondition()
	{
		return SubConditionSQL.IN;
	}
}
