package com.logntw.eva.sql.dialect;

import com.logntw.eva.jdbc.JDBCClazz;


public class PostgreSQLDialect extends Dialect
{
	public final static PostgreSQLDialect INSTANCE = new PostgreSQLDialect();
	public PostgreSQLDialect()
	{
		indexable( JDBCClazz.BIT, "bool" );
		indexable( JDBCClazz.BIGINT, "int8" );
		indexable( JDBCClazz.SMALLINT, "int2" );
		indexable( JDBCClazz.TINYINT, "int2" );
		indexable( JDBCClazz.INTEGER, "int4" );
		indexable( JDBCClazz.CHAR, "char(1)" );
		indexable( JDBCClazz.VARCHAR, "varchar(N)" );
		indexable( JDBCClazz.FLOAT, "float4" );
		indexable( JDBCClazz.DOUBLE, "float8" );
		indexable( JDBCClazz.DATE, "date" );
		indexable( JDBCClazz.TIME, "time" );
		indexable( JDBCClazz.TIMESTAMP, "timestamp" );
		indexable( JDBCClazz.VARBINARY, "bytea" );
		notIndexable( JDBCClazz.LONGVARCHAR, "text" );
		notIndexable( JDBCClazz.LONGVARBINARY, "bytea" );
		notIndexable( JDBCClazz.CLOB, "text" );
		notIndexable( JDBCClazz.BLOB, "oid" );
		indexable( JDBCClazz.NUMERIC, "numeric(N,M)" );
		indexable( JDBCClazz.OTHER, "uuid" );
	}
	
	@Override
	public String name()
	{		
		return "postgresql";
	}
	
	@Override
	public String getIdentityColumnString()
	{
		return "serial not null";		
	}	
	
	@Override
	public boolean supportsLimit() 
	{
		return true;
	}
	
	@Override
	public StringBuilder getLimitStringEnd(StringBuilder sqlStmt) 
	{
		return sqlStmt.append(" limit ? offset ?");			
	}
	
	@Override
	public boolean bindLimitParametersInReverseOrder() 
	{
		return true;
	}

	@Override
	public String getWriteLockString(int timeout) 
	{
		if ( timeout < 0 )
			return " for update nowait";
		else
			return " for update";
	}
	
	@Override
	public String getReadLockString(int timeout) 
	{
		if ( timeout < 0 )
			return " for share nowait";
		else
			return " for share";
	}
}
